unit fm_i332Buffer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fm_i332, StdCtrls, ComCtrls, ExtCtrls, Buttons, i332_def;

CONST IPLBufferSize = 200;    // Wortgre des IPLBuffers
                              // Die Gre des Buffers ist nicht kritisch; da
                              // fr dieses Beispiel jedoch auf Fehler-Abfragen
                              // (insb. "Buffer voll") verzichtet wird, sollte
                              // er mindestens so gro sein, da er die Befehle
                              // fr zwei Linear-Interpolationsstze aufnehmen
                              // kann; pro Satz sind erforderlich:
                              // 8x Write2(8x Distanz): 8x3 Worte
                              // 1x Write2(Vorschub):     3 Worte
                              // 1x Write0(IPL-Befehl):   1 Wort
                              // zusammen:               28 Worte
type
  TfmI332Buffer = class(TfmI332)
    btnIPL: TBitBtn;
    Timer1: TTimer;
    rgI332Disp: TRadioGroup;
    lblPos: TLabel;
    rgI332IPL: TRadioGroup;
    procedure Timer1Timer(Sender: TObject);
    procedure btnIPLClick(Sender: TObject);
  private
    { Private-Deklarationen }

    // Buffer und I332BufHandles fr das Anzeige-Beispiel
    I332DspHandle_app: INTEGER;
    I332DspHandle_dll: INTEGER;
    I332DspBuffer:     TAxisArray;

    // Buffer und I332BufHandles fr das Interpolationsbeispiel
    I332IPLHandle_app: INTEGER;
    I332IPLHandle_dll: INTEGER;
    I332IPLBuffer:     ARRAY[0..IPLBufferSize-1] OF WORD;

  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

implementation

uses i332_ifc;

{$R *.dfm}

//------------------------------------------------------------------------------
// I332BufHandles (global) erzeugen:
//------------------------------------------------------------------------------
constructor TfmI332Buffer.Create(AOwner: TComponent);
BEGIN
  inherited Create(AOwner);

  I332DspHandle_app := I332InitBuffer(I332Handle,
                                      @I332DspBuffer,  // Langwort-Array fr 8(=Axes) Achsen
                                      Axes * 2,        // Wortgre
                                      FALSE);
  I332DspHandle_dll := I332InitBuffer(I332Handle,
                                      NIL,             // DLL-internen Buffer erzeugen
                                      Axes * 2,        // Wortgre
                                      FALSE);

  I332IPLHandle_app := I332InitBuffer(I332Handle,
                                      @I332IPLBuffer,  // Wort-Array fr die Interpolationsbefehle
                                      IPLBufferSize,   // Wortgre
                                      TRUE);           // nur W0/W1/W2-Befehle
  I332IPLHandle_dll := I332InitBuffer(I332Handle,
                                      NIL,             // DLL-internen Buffer erzeugen
                                      IPLBufferSize,   // Wortgre
                                      TRUE);           // nur W0/W1/W2-Befehle
END;

//------------------------------------------------------------------------------
// I332BufHandles wieder freigeben
//------------------------------------------------------------------------------
destructor TfmI332Buffer.Destroy;
BEGIN
  I332FreeBuffer(I332DspHandle_app);
  I332FreeBuffer(I332DspHandle_dll);

  I332FreeBuffer(I332IPLHandle_app);
  I332FreeBuffer(I332IPLHandle_dll);

  inherited Destroy;
END;

//------------------------------------------------------------------------------
// Buffer-Demo: Achs-Anzeige
//------------------------------------------------------------------------------

procedure TfmI332Buffer.Timer1Timer(Sender: TObject);

  PROCEDURE DispAxes(CONST AxArray: TAxisArray);
  // Hilfsfunktion: Bildschirmdarstellung
  VAR s: String;
      i: INTEGER;
  BEGIN
    s := '';
    FOR i := 0 TO Axes-1 DO s := s + IntToStr(AxArray[i]) + #13;
    lblPos.Caption := s;
  END;

  // ---------------------------------------------------------------------------
  // Die einzelnen Varianten:
  // ---------------------------------------------------------------------------

  // ---------------------------------------------------------------------------
  // einfachste Variante: Aufruf der DLL-Funktion I332GetAxArray
  // I332GetAxArray verwendet intern ebenfalls einen Buffer.
  // ---------------------------------------------------------------------------
  PROCEDURE DispA;
  VAR AxArray: TAxisArray;
  BEGIN
    I332GetAxArray(I332Handle,icGetPos,AxArray);
    DispAxes(AxArray);
  END;

  // ---------------------------------------------------------------------------
  // Eintrag der Befehle in einen lokalen Buffer (hier: AxArray)
  // Aufruf der DLL-Funktion I332XmtBuffer
  // ---------------------------------------------------------------------------
  PROCEDURE DispX;
  VAR AxArray: TAxisArray;
      i: INTEGER;
  BEGIN
    FOR i := 0 TO Axes-1
    DO AxArray[i] := icGetPos+i*256;  // AxArray[i] = Longword --> Loword = icGetPos, Hiword = 0
    I332XmtBuffer(I332Handle,@AxArray,Axes*2);
    DispAxes(AxArray);
  END;

  // ---------------------------------------------------------------------------
  // Aufruf der DLL-Funktion I332Read2 zur Abfrage jeder einzelnen Position
  // nicht effizient fr USB-Version (s.Handbuch)
  // ---------------------------------------------------------------------------
  PROCEDURE Disp;
  VAR AxArray: TAxisArray;
      i: INTEGER;
  BEGIN
    FOR i := 0 TO Axes-1
    DO I332Read2(I332Handle,icGetPos+i*256,AxArray[i]);
    DispAxes(AxArray);
  END;

  // ---------------------------------------------------------------------------
  // Aufruf der DLL-Funktion I332Read2, dabei ist I332DspHandle_app ein Handle
  // auf den globalen Buffer I332DspBuffer;
  // I332FlushBuffer fhrt die Befehle aus, die Ergebnisse stehen anschlieend
  // im I332DspBuffer.
  // ---------------------------------------------------------------------------
  PROCEDURE Disp_app;
  VAR i: INTEGER;
      dummy: LONGINT;
  BEGIN
    I332ClearBuffer(I332DspHandle_app); // sicherheitshalber
    FOR i := 0 TO Axes-1
    DO I332Read2(I332DspHandle_app,icGetPos+i*256,dummy);
    I332FlushBuffer(I332DspHandle_app); // ausfhren
    DispAxes(I332DspBuffer);
  END;

  // ---------------------------------------------------------------------------
  // Aufruf der DLL-Funktion I332Read2, dabei ist I332DspHandle_dll ein Handle
  // auf einen DLL-internen Buffer;
  // I332FlushBuffer fhrt die Befehle aus, die Ergebnisse stehen anschlieend
  // im DLL-internen Buffer und mssen per I332GetBuffer in einen lokalen
  // Buffer kopiert werden.
  // --> fr diesen Zweck sehr umstndlich
  // ---------------------------------------------------------------------------
  PROCEDURE Disp_dll;
  VAR AxArray: TAxisArray;
      i: INTEGER;
      dummy: LONGINT;
  BEGIN
    I332ClearBuffer(I332DspHandle_dll);
    FOR i := 0 TO Axes-1
    DO I332Read2(I332DspHandle_dll,icGetPos+i*256,dummy);
    I332FlushBuffer(I332DspHandle_dll);
    I332GetBuffer(I332DspHandle_dll,@AxArray,Axes*2,0);
    DispAxes(AxArray);
  END;

begin
  CASE rgI332Disp.ItemIndex OF
    0: Disp;
    1: DispA;
    2: DispX;
    3: Disp_app;
    4: Disp_dll;
    ELSE DispA;
  END;
end;

//------------------------------------------------------------------------------
// Buffer-Demo: Linear-Interpolation
//------------------------------------------------------------------------------

// Daten fr den Interpoaltionssatz:
VAR AxArray: TAxisArray = (1000,2000,3000,4000,5000,6000,7000,8000);
CONST Vorschub = 100;

procedure TfmI332Buffer.btnIPLClick(Sender: TObject);

  // ---------------------------------------------------------------------------
  // Die einzelnen Varianten:
  // ---------------------------------------------------------------------------

  // ---------------------------------------------------------------------------
  // UP fr die Varianten Nr. 0,2,3:
  // Aufruf der DLL-Funktion I332Write.. zur bergabe der Interpolationsdaten
  // AI332Handle = I332Handle oder I332BufHandle:
  // - Variante Nr. 0 --> AI332Handle = kein BufferHandle:
  //   nicht effizient fr USB-Version (s.Handbuch)
  // - Variante Nr. 2,3 --> AI332Handle = BufferHandle:
  //   anschlieend I332FlushBuffer notwendig
  // ---------------------------------------------------------------------------
  PROCEDURE DoIPLLin(AI332Handle: LONGINT);
  // DoIPLLin berprft NICHT, ob genug Platz in der I332 ist
  VAR i: INTEGER;
  BEGIN
    I332Write2(AI332Handle,icSetVCmd,Vorschub);
    FOR i := 0 TO Axes-1 DO I332Write2(AI332Handle,icSetDist+i*256,AxArray[i]);
    I332Write0(AI332Handle,icIPLLin);
  END;

  // ---------------------------------------------------------------------------
  // Variante Nr. 1:
  // Eintrag der Befehle in einen lokalen Buffer (hier: IPLBuffer)
  // Aufruf der DLL-Funktion I332XmtBuffer
  // DoIPLLinX berprft NICHT, ob genug Platz in der I332 ist
  // ---------------------------------------------------------------------------
  PROCEDURE DoIPLLinX;
  VAR IPLBuffer: ARRAY[0..30] OF WORD; // 9x Write2(8x Distanz, 1x Vorschub): 9x3 Worte
                                       // + 1x Write0(IPL-Befehl): 1 Wort
                                       // --> zusammen 28 Worte
      i,j: INTEGER;
  BEGIN
    j := 0;

    IPLBuffer[j+0] := icSetVCmd;
    IPLBuffer[j+1] := LoWord(Vorschub);
    IPLBuffer[j+2] := HiWord(Vorschub);
    Inc(j,3);

    FOR i := 0 TO Axes-1 DO BEGIN
      IPLBuffer[j+0] := icSetDist+i*256;
      IPLBuffer[j+1] := LoWord(AxArray[i]);
      IPLBuffer[j+2] := HiWord(AxArray[i]);
      Inc(j,3);
    END;

    IPLBuffer[j+0] := icIPLLin;
    Inc(j);

    I332XmtBuffer(I332Handle,@IPLBuffer,j);
  END;

  // ---------------------------------------------------------------------------
  // Variante Nr. 4:
  // einfachste Variante: Aufruf der DLL-Funktion I332DoIPLLin
  // nicht ganz so effizient fr USB-Version, da I332DoIPL... zuvor einmal den
  // I332-Status abfragt, ob noch Platz fr weitere Stze ist.
  // Fr die bertragung der Satzdaten wird intern jedoch ebenfalls ein Buffer
  // verwendet.
  // ---------------------------------------------------------------------------

  // ---------------------------------------------------------------------------
  // Varianten 5,6:
  // Aufruf der DLL-Funktion I332DoIPLLin mit I332BufHandle
  // effizienter fr USB-Version, da I332DoIPL... jetzt NICHT den I332-Status abfragt
  // Die Applikation mu VOR dem Aufruf SELBST sicherstellen, da noch Platz
  // fr weitere Stze ist.
  // ---------------------------------------------------------------------------

VAR i: INTEGER;
begin
  CASE rgI332IPL.ItemIndex OF
    0: DoIPLLin(I332Handle);
    1: DoIPLLinX;

    2: BEGIN
         I332ClearBuffer(I332IPLHandle_app);                // sicherheitshalber
         // den ganzen Satz z.B. 2x:
         DoIPLLin(I332IPLHandle_app);
         DoIPLLin(I332IPLHandle_app);
         I332FlushBuffer(I332IPLHandle_app);                // hier erfolgt erst die bertragung
       END;

    3: BEGIN
         I332ClearBuffer(I332IPLHandle_dll);                // sicherheitshalber
         // den ganzen Satz z.B. 2x:
         DoIPLLin(I332IPLHandle_dll);
         DoIPLLin(I332IPLHandle_dll);
         I332FlushBuffer(I332IPLHandle_dll);                // hier erfolgt erst die bertragung
       END;

    4: // I332Handle ist kein BufferHandle:
       // --> I332DoIPLLin berprft, ob genug Platz in der I332 ist und liefert
       //     ggf. eine Fehlermeldung zurck (die hier im Beispeil aber ignortiert wird...)
       I332DoIPLLin(I332Handle,8,AxArray,Vorschub,fRel);

    5: // I332Handle ist ein BufferHandle:
       // --> I332DoIPLLin berprft NICHT, ob genug Platz in der I332 ist
       BEGIN
         I332ClearBuffer(I332IPLHandle_app);                // sicherheitshalber
         // den ganzen Satz z.B. 2x:
         I332DoIPLLin(I332IPLHandle_app,8,AxArray,Vorschub,fRel);
         I332DoIPLLin(I332IPLHandle_app,8,AxArray,Vorschub,fRel);
         I332FlushBuffer(I332IPLHandle_app);                // hier erfolgt erst die bertragung
       END;

    6: // I332Handle ist ein BufferHandle:
       // --> I332DoIPLLin berprft NICHT, ob genug Platz in der I332 ist
       BEGIN
         I332ClearBuffer(I332IPLHandle_dll);                // sicherheitshalber
         // den ganzen Satz z.B. 2x:
         I332DoIPLLin(I332IPLHandle_dll,8,AxArray,Vorschub,fRel);
         I332DoIPLLin(I332IPLHandle_dll,8,AxArray,Vorschub,fRel);
         I332FlushBuffer(I332IPLHandle_dll);                // hier erfolgt erst die bertragung
       END;
  END;

  // Beim nchsten Click in die andre Richtung fahren:
  // alles invertieren
  FOR i := 0 TO Axes-1 DO AxArray[i] := -AxArray[i];
end;

end.
