unit i332_cnc;

interface

uses ExtThread, i332_def;

TYPE TI332CNC = class(TObject)
       constructor Create(AI332Handle: LONGINT);
       PROCEDURE ScanLine(DINLine: String);
       FUNCTION UpdateI332: INTEGER;
     private
       I332Handle:  LONGINT;
       DINAdressen: ARRAY['A'..'Z'] OF LONGINT;
       Wege:        TAxisArray;
       Parameter:   TAxisArray;
       AxLbls:      TAxLbls;
       Mode:        INTEGER;
       Vorschub:    INTEGER;
     END;

TYPE TI332Thread = class(TExtThread)
       I332CNC: TI332CNC;
       constructor Create(AI332CNC: TI332CNC);
       PROCEDURE Execute; override;
     END;

implementation

uses SysUtils, WinUtils,
     Classes,
     i332_ifc;

{********************************************************************}

constructor TI332CNC.Create(AI332Handle: LONGINT);
BEGIN
  inherited Create;
  I332Handle := AI332Handle;
  I332GetAxLabels(I332Handle,AxLbls);
  Mode := fRel;
END;

PROCEDURE TI332CNC.ScanLine(DINLine: String);

  PROCEDURE DelBlanks (VAR s : String);
  BEGIN
    WHILE Pos (' ',s) <> 0 DO Delete(s,Pos(' ',s),1);
  END;

  FUNCTION SatzAdr (c : CHAR; VAR s: String): Extended;
  VAR h: Extended;
      s1: String;
      e: INTEGER;
  BEGIN
    IF (s <> '') AND (c = s[1]) AND (c IN ['A'..'Z']) THEN BEGIN
      Delete(s,1,1);
      Val(s,h,e);
      IF e <> 0 THEN BEGIN
        s1 := Copy(s,1,e-1);
        Delete(s,1,e-1);
        Val(s1,h,e);
      END ELSE s := '';
      SatzAdr := h;
    END ELSE SatzAdr := undefL;
  END;

VAR c: CHAR;
    h: Extended;
BEGIN
  FOR c := 'A' TO 'Z' DO DINAdressen[c] := UndefL;
  DelBlanks(DINLine);
  WHILE Length(DINLine) > 1 DO BEGIN
    DINLine[1] := UpCase(DINLine[1]);
    c := DINLine[1];
    h := SatzAdr(c,DINLine);
    IF h <> undefL THEN CASE c OF
      'N','G','F','M','T','S','D','R': DINAdressen[c] := Round(h);
      ELSE DINAdressen[c] := Round(h*1000);
    END; {CASE}
  END; {WHILE}
END;

FUNCTION TI332CNC.UpdateI332: INTEGER;
VAR i : INTEGER;
BEGIN
  Result := 0;
  FOR i := 0 TO Axes-1 DO Wege[i] := DINAdressen[AxLbls[i]];
  IF DINAdressen['F'] <> undefL THEN Vorschub := DINAdressen['F'];

  // CNC-Satz bertragen:
  CASE DINAdressen['G'] OF
    0: Result := I332DoIPLLin(I332Handle,Axes,Wege,FEilgang,Mode);
    1: Result := I332DoIPLLin(I332Handle,Axes,Wege,Vorschub,Mode);
    2: Result := I332DoIPLcw(I332Handle,Axes,Wege,Vorschub,Mode,0,1,DINAdressen['I'],DINAdressen['J']);
    3: Result := I332DoIPLccw(I332Handle,Axes,Wege,Vorschub,Mode,0,1,DINAdressen['I'],DINAdressen['J']);
   90: Mode := fAbs;
   91: Mode := fRel;
   92: FOR i := 0 TO Axes-1 DO IF Wege[i] <> UndefL
       THEN Result := I332SetAchsReg(I332Handle,icSetPos,i,Wege[i]);
   74: FOR i := 0 TO Axes-1 DO IF Wege[i] <> UndefL
       THEN Result := I332DoAxisRef(I332Handle,i);
  END;
END;

{********************************************************************}

constructor TI332Thread.Create(AI332CNC: TI332CNC);
BEGIN
  inherited Create(tcSuspended);
  I332CNC := AI332CNC;
  Resume;
END;

PROCEDURE TI332Thread.Execute;
VAR i: INTEGER;
    status: INTEGER;
    sfstatus: WORD;
BEGIN
  try
    WITH TStringList.Create DO try
      LoadFromFile('F:\temp\9042-NEU.NC');
      i := 0;
      WHILE NOT Terminated AND (i < Count) DO BEGIN

        // CNC-Zeile analysieren:
        I332CNC.ScanLine(Strings[i]);

        // ggf. Warten bis I332 Platz fr neue Stze hat:
        WITH I332CNC DO
        CASE DINAdressen['G'] OF
          0,1,2,3: REPEAT
                     I332GetStatus(I332Handle,status);
                     IF (status and stOVF) = 0
                     THEN Break;
                     {ELSE} Sleep(100);
                   UNTIL FALSE OR Terminated;
          74,92:   REPEAT
                     I332GetStatus(I332Handle,status);
                     I332GetSFStatus(I332Handle,sfstatus);
                     IF ( (status and stBNEF) = 0 )       // IPL-Buffer leer
                     AND ( sfstatus = 0 )                 // keine Sonderfunktion aktiv
                     THEN Break;
                     {ELSE} Sleep(100);
                   UNTIL FALSE OR Terminated;
        END;
        IF Terminated THEN Break;

        // CNC-Satz bertragen:
        IF I332CNC.UpdateI332 <> 0 THEN ;// Fehlerbehandlung

        Inc(i);
      END;
    finally
      Free;
    END;
  except
    ON E: Exception DO ErrorMsg(E.Message);
  end;
END;

{********************************************************************}

end.

